<?php
class ControllerExtensionPaymentCryptexy extends Controller {

    private $generateUrl = 'https://merchants.cryptexy.com/api/v1/invoice/generate';

    private $showInvoiceUrl = 'https://merchants.cryptexy.com/api/v1/invoice/show/';
    private static $valid_currencies = array(
        'RUB', 'RUR'
    );
    public function index() {
		$this->load->language('extension/payment/cryptexy');

		$data['button_confirm'] = $this->language->get('button_confirm');

        $data['action'] = $this->url->link('extension/payment/cryptexy/pay', '', true);

        $this->session->data['from_cryptexy'] = true;

        return $this->load->view('extension/payment/cryptexy', $data);

    }

    protected function getCurrencyCode() {
        foreach (self::$valid_currencies as $code) {
            if ($this->currency->has($code)) {
                return $code;
            }
        }
        die(sprintf('Currency code (%s) not found', implode(', ', self::$valid_currencies)));
    }

	public function pay() {
		$this->load->language('extension/payment/cryptexy');


            if ($this->request->server['REQUEST_METHOD'] == 'POST' && isset($this->session->data['from_cryptexy'])) {

                unset($this->session->data['from_cryptexy']);
                $this->load->model('checkout/order');
                $this->load->model('extension/payment/cryptexy');

                $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

                if ($order_info) {
                    $fields['api_key'] = $this->config->get('payment_cryptexy_api_key');

                    $fields['extra']['store_name'] = html_entity_decode($this->config->get('config_name'), ENT_QUOTES, 'UTF-8');

                    $fields['items'] = array();

                    foreach ($this->cart->getProducts() as $product) {

                        $fields['items'][] = array(
                            'item_id' => htmlspecialchars($product['product_id']),
                            'item_name' => htmlspecialchars($product['name']),
                            'item_price' => $this->currency->format($product['price'], $order_info['currency_code'], false, false),
                            'item_amount' => $product['quantity']
                        );
                    }


                    $fields['extra']['currency_code'] = $order_info['currency_code'];

                    $fields['extra']['first_name'] = html_entity_decode($order_info['payment_firstname'], ENT_QUOTES, 'UTF-8');
                    $fields['extra']['last_name'] = html_entity_decode($order_info['payment_lastname'], ENT_QUOTES, 'UTF-8');
                    $fields['extra']['address1'] = html_entity_decode($order_info['payment_address_1'], ENT_QUOTES, 'UTF-8');
                    $fields['extra']['address2'] = html_entity_decode($order_info['payment_address_2'], ENT_QUOTES, 'UTF-8');
                    $fields['extra']['city'] = html_entity_decode($order_info['payment_city'], ENT_QUOTES, 'UTF-8');
                    $fields['extra']['zip'] = html_entity_decode($order_info['payment_postcode'], ENT_QUOTES, 'UTF-8');
                    $fields['extra']['country'] = $order_info['payment_iso_code_2'];
                    $fields['customer_email'] = $order_info['email'];
                    $fields['comeback_url'] = $this->url->link('checkout/success');
                    $fields['paid_callback'] = $this->url->link('extension/payment/cryptexy/callback', '', true);

                    $currency_code = $this->getCurrencyCode();

                    $fields['total_price'] = $this->currency->convert($order_info['total'], $this->config->get('config_currency'), $currency_code);

                    $fields['custom_id'] = $this->session->data['order_id'];


                    $fields['extra']['invoice'] = $this->session->data['order_id'] . ' - ' . html_entity_decode($order_info['payment_firstname'], ENT_QUOTES, 'UTF-8') . ' ' . html_entity_decode($order_info['payment_lastname'], ENT_QUOTES, 'UTF-8');

                    $this->model_extension_payment_cryptexy->logger($fields);
                    $response_data = $this->model_extension_payment_cryptexy->sendCurl($this->generateUrl, $fields);

                    if ($response_data && strtolower($response_data->status) == 'success') {
                        $this->model_checkout_order->addOrderHistory($this->session->data['order_id'],
                                $this->config->get('payment_cryptexy_pending_status_id'),
                                'Cryptexy: Waiting Payment',
                                0);

                        $this->model_extension_payment_cryptexy->logger($response_data);
                        $this->model_extension_payment_cryptexy->updateOrderInfo($this->session->data['order_id'],$response_data->data->verification_code);
                        $this->model_extension_payment_cryptexy->logger('order updated');
                        $this->response->redirect($this->showInvoiceUrl . $response_data->data->invoice_code);

                    } elseif ($response_data) {
                        $this->model_extension_payment_cryptexy->logger($response_data);
                        $this->model_checkout_order->addOrderHistory($this->session->data['order_id'],
                            $this->config->get('payment_cryptexy_failed_status_id'),
                            'Cryptexy: Payment Failed',
                            0);
                        $this->failure($response_data->message);
                    } else {
                        $this->model_extension_payment_cryptexy->logger('no response');
                        $this->model_checkout_order->addOrderHistory($this->session->data['order_id'],
                            $this->config->get('payment_cryptexy_failed_status_id'),
                            'Cryptexy: Payment Failed',
                            0);
                        $this->response->redirect($this->url->link('checkout/failure', '', true));
                    }

                }

            }

		}

    public function failure($text='') {
        $this->load->language('checkout/failure');

        $this->document->setTitle($this->language->get('heading_title'));

        $data['breadcrumbs'] = array();

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_home'),
            'href' => $this->url->link('common/home')
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_basket'),
            'href' => $this->url->link('checkout/cart')
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_checkout'),
            'href' => $this->url->link('checkout/checkout', '', true)
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_failure'),
            'href' => $this->url->link('checkout/failure')
        );

        $data['text_message'] = $text;

        $data['continue'] = $this->url->link('common/home');

        $data['column_left'] = $this->load->controller('common/column_left');
        $data['column_right'] = $this->load->controller('common/column_right');
        $data['content_top'] = $this->load->controller('common/content_top');
        $data['content_bottom'] = $this->load->controller('common/content_bottom');
        $data['footer'] = $this->load->controller('common/footer');
        $data['header'] = $this->load->controller('common/header');

        $this->response->setOutput($this->load->view('common/success', $data));
    }
    public function callback() {
        $this->load->model('checkout/order');
        $this->load->model('extension/payment/cryptexy');
        $this->model_extension_payment_cryptexy->logger('callback');
        $this->model_extension_payment_cryptexy->logger($this->request->get);
        $this->model_extension_payment_cryptexy->logger($this->request->post);
        if (isset($this->request->post['verification_code'])) {
            $this->model_extension_payment_cryptexy->logger('We have verification code');
            $order_info = $this->model_extension_payment_cryptexy->getOrderByCode($this->request->post['verification_code']);
            if ($order_info) {
                $this->model_extension_payment_cryptexy->logger('We have this order and verification code is correct');
                if ((float)$this->request->post['deviation'] == 0) {
                    $this->model_checkout_order->addOrderHistory($order_info['order_id'], $this->config->get('payment_cryptexy_completed_status_id'),
                        'Cryptexy: Полная оплата', 0);
                } elseif ((float)$this->request->post['deviation'] >= 0) {
                    $this->model_checkout_order->addOrderHistory($order_info['order_id'], $this->config->get('payment_cryptexy_more_status_id'),
                        'Cryptexy: Переплата', 0);
                } elseif ((float)$this->request->post['deviation'] < 0) {
                    $this->model_checkout_order->addOrderHistory($order_info['order_id'], $this->config->get('payment_cryptexy_less_status_id'),
                        'Cryptexy: Неполная оплата', 0);
                } else {
                    $this->model_checkout_order->addOrderHistory($order_info['order_id'], $this->config->get('payment_cryptexy_failed_status_id'),
                        'Cryptexy: разница не число!', 0);
                }
                echo "ok";
                $this->model_extension_payment_cryptexy->logger('ok');
            } else {
                $this->model_extension_payment_cryptexy->logger('No order with this verification code');

            }



        } else {
            $this->model_extension_payment_cryptexy->logger('No verification code');
            $this->model_extension_payment_cryptexy->logger($this->request->post);
        }
    }
}